# File generated from our OpenAPI spec by Stainless.

from typing_extensions import Literal

__all__ = ["SpendLimitDuration"]

SpendLimitDuration = Literal["ANNUALLY", "FOREVER", "MONTHLY", "TRANSACTION"]
